#ifndef _FLE_IMAGE_H
#define _FLE_IMAGE_H

#include <string.h>

#include "EDE.h"
#include "FLE_Renderer.h"
#include "FLE_List.h"

#include <fltk/Fl_Image.h>

class FLE_Image;

typedef struct _reader
{
    bool  (*is_valid)(void *stream, bool file);
    FLE_Image *(*create)(void *stream, int size, bool file);
} ImageReader;

typedef enum
{
	MASK_NONE = 0,
	MASK_ALPHA,
	MASK_COLORKEY,
	MASK_PIXELKEY
} Mask_Type;

 /*!
 			IMAGE
 */
class FLE_Image : public Fl_Image {
    void init(int W, int H, int bits_pp, uint8 *data, uint32 Rmask, uint32 Gmask, uint32 Bmask, uint32 Amask, int flags);

public:
    //If you provide data, you may want to provide also data size, otherwise it's sizeof(data)
    static FLE_Image *read(const char *filename, const uint8 *data = 0, uint data_size=0);

		/*! 
			@param W As a width of image
			@param H As a height of image
			@param bits_pp As a bits per pixel (8,15,16,24,32)
			@param data As pre-allocated data of image, or 0 (allocated by FLE_Image)			
			
			You may also want to provide color masks, e.g. ARGB 32-bit image they are Rmask=0x00FF0000, Gmask=0x0000FF00, Bmask=0x000000FF, Amask=0xFF000000			
		*/
    FLE_Image(int W, int H, int bits_pp, uint8 *data, uint32 Rmask, uint32 Gmask, uint32 Bmask, uint32 Amask, int flags);
		/*! 
			@param W As a width of image
			@param H As a height of image
			@param bits_pp As a bits per pixel (8,15,16,24,32)			
			
			Simplier contructor, data is allocated always by FLE_Image.			
		*/
    FLE_Image(int W, int H, int bits_pp=32, int flags=0);

    // Copies image 'i' to this (NOT WORKING!)
    //FLE_Image(FLE_Image &i);

		/*!
			Destructor, cleans image. Deletes all mask and offscreen pixmaps.
			Deallocates data.
		*/
    virtual ~FLE_Image();

		/*!
			@param W As a reference integer, Width is stored to this.
			@param H As a reference integer, Heigth is stored to this.
			
			Get image width and heigth. Fl_Image overloaded function. 
			
			@see width() and height()
		*/
    void measure(int &W, int &H) { W=w; H=h;}

    /*!
    	Draws image to screen. 
    	
    	<i>dx, dy, dw, dh</i> are destination coordinates in widget to draw.<br>
    	<i>sx, sy, dw, sh</i> are source coordinates in image.
    */
    void draw(int dx, int dy, int dw, int dh,
              int sx, int sy, int sw, int sh,
              Fl_Flags f);
		/*!
    	Draws image to screen. Fl_Image override function.
    	
    	<i>x, y, w, h</i> are destination coordinates in widget to draw.
    */
    virtual void draw(int x, int y, int w, int h, Fl_Flags f = 0) { draw(x,y,w,h,0,0,0,0,f); }

    // Mask handling
    Mask_Type mask_type() { return _masktype; }
    void mask_type(Mask_Type val) { _masktype = val; }
    void set_mask(Pixmap m);
		Pixmap get_mask() { return (Pixmap)mask; }
    Pixmap get_offscreen() { return (Pixmap)id; }

    // Creates maskes.. uses automaticly by mask_type
    Pixmap create_alpha_mask(int X, int Y, int W, int H, uint8 threshold);
    Pixmap create_color_mask(int X, int Y, int W, int H, uint32 color);
    Pixmap create_pixel_mask(int X, int Y, int W, int H, uint32 pixel);		

    // Scales image to new image. SLOW (Only very little asm optimization)
    FLE_Image *scale(int W, int H);

    /*! 
    	Converts image to grayscale.
    
    	Default argument makes new image to be system format
    		
    	Supported depths in format are <b>15, 16, 24, 32</b>
    */
    FLE_Image *grayscale(FLE_PixelFormat *new_format=FLE_Renderer::system_format());

    /*! 
    	Blends image to new image, using <i>color</i> as foregroud (top of it)
    	
    	Default argument makes new image to be system format   		
   		
   		Supported depths in format are <b>15, 16, 24, 32</b>    	
    */
    FLE_Image *fore_blend(uint color, FLE_PixelFormat *new_format=FLE_Renderer::system_format());

    /*! 
    	Blends image to new image, using <i>color</i> as backgroud
    	
    	Default argument makes new image to be system format   		
    	
    	Supported depths in format are <b>15, 16, 24, 32</b>    	
    */
    FLE_Image *back_blend(uint color, FLE_PixelFormat *new_format=FLE_Renderer::system_format());

    // Blends image to new image, using back as background
    // Default makes new image to be system fmt(no need to convert)
    // supported bits_per_pixels = 15, 16, 24, 32
    FLE_Image *blend(FLE_Image *back, FLE_Rect *back_rect, FLE_PixelFormat *new_format=FLE_Renderer::system_format());
    FLE_Image *blend(FLE_Image *back, int x, int y, int w, int h, FLE_PixelFormat *new_format=FLE_Renderer::system_format()) {
		FLE_Rect tmp(x,y,w,h); return blend(back, &tmp, new_format);
	}

    // Frees all allocated buffers
    void clear();

    // Makes offscreen pixmaps redraw
    void invalidate();

    void system_convert();

    // Image data
    inline uint8 *data() { return _data; }

    int width()  { return w; }
    int height() { return h; }

    int pitch() { if(!_pitch) _pitch = FLE_Renderer::calc_pitch(fmt.bytespp, w); return _pitch; }
    inline int bytespp() { return fmt.bytespp; 	} //Bytes per pixel
    inline int bitspp() 	{ return fmt.bitspp; 	}	//Bits per pixel

    int state() { return _state; }
    void state(int s) { _state = s; }

    // Flag handling
    int  flags() 							 { return _flags; 	 }
    void flags(int flags) 		 { _flags = flags;  }
    void set_flags(int flags)	 { _flags |= flags; }
    void clear_flags(int flags){ _flags &= ~flags;}
    bool isset(int flags) 		 { return ((_flags & (unsigned)flags) == (unsigned)flags); }

    // For colorkey blitting and masking (colorkey in RGBA)
    uint32 colorkey() { return fmt.colorkey; }
    void colorkey(uint32 c) { fmt.colorkey = c; }

    // 0 - 255 Surface alpha
    uint8 alpha() { return fmt.alpha; }
    void alpha(uint8 a) { fmt.alpha = a; }

    void threshold(uint8 t) { _threshold = t; }
    uint8 threshold() { return _threshold; }

    // NOT WORKING!
    FLE_Colormap *colormap() { return fmt.palette; }
    FLE_Colormap *colormap(FLE_Colormap *new_map) { FLE_Colormap *old = fmt.palette; fmt.palette = new_map; return old; }

    FLE_PixelFormat *format() { return &fmt; }
    bool check_map(FLE_PixelFormat *cur_fmt, FLE_PixelFormat *new_fmt);
    bool check_map(FLE_PixelFormat *new_fmt);

    bool auto_scale() { return _auto_scale; }
    void auto_scale(bool v) { _auto_scale = v; }

    static void register_reader(ImageReader *reader);
    static void unregister_reader(ImageReader *reader);
    static ImageReader *find_reader(ImageReader *reader);

protected:
    void to_screen(int X, int Y, int W, int H, int cx, int cy);

    FLE_PixelFormat fmt;

    int _state;

    Mask_Type _masktype;

    unsigned _flags;
    int _pitch;

    uint8 *_data;
    uint8 _threshold;

    bool _auto_scale;
	int last_w, last_h; //last scaled size

    // Data store for selected, inactive...	
    FLE_Image *_mod_data;
};

class FLE_ImageCache
{
	typedef struct {
		FLE_Image *image;
		char *identify;
	} CachedImage;

	FLE_List<CachedImage> images;
	int size_;
	bool autodelete_;
public:
	// Size of cache as a param, 0 is unlimited
	FLE_ImageCache(int size=0) { size_ = size; autodelete_ = false; }
	~FLE_ImageCache() { clear(); }

	// Clears cache, deletes images if auto delete is set true
	void clear() {
		for(CachedImage *i=images.first(); i!=0; i=images.next()) {
			if(i->identify) delete []i->identify;
			if(i->image && autodelete_) delete i->image;
			delete i;
		}
		images.clear();
	}
	FLE_Image *get(const char *identify) {
		for(CachedImage *i=images.first(); i!=0; i=images.next()) {
			if(!strcmp(identify, i->identify)) return i->image;
		}
		return 0;
	}

	// Removes image from cache, returns removed image
	FLE_Image *remove(const char *identify) {
		for(CachedImage *i=images.first(); i!=0; i=images.next()) {
			if(!strcmp(identify, i->identify)) {
				images.remove(i);
				if(i->identify) delete []i->identify;
				FLE_Image *tmp = i->image;
				delete i;
				return tmp;
			}
		}
		return 0;
	}

	// Returns replaced image, if any
	FLE_Image *add(FLE_Image *i, const char *identify) {
		if(!i) return 0;

		CachedImage *c;
		// Remove previously cached with same identify
		for(c=images.first(); c!=0; c=images.next()) {
			if(!strcmp(identify, c->identify)) {				
				if(c->identify) delete []c->identify;
				c->identify = strdup(identify);
				FLE_Image *tmp = c->image;
				c->image = i;
				return tmp;
			}
		}
		
		c = new CachedImage;
		c->image = i;
		c->identify = strdup(identify);

		images.append(c);
		check_size();

		return 0;
	}

	// Check cache size and removes images first, if needed
	void check_size() {
		if(size_>0 && images.count() > size_) {
			CachedImage *i = images.first();
			images.remove(i);
			if(i->identify) delete []i->identify;
			if(i->image && autodelete_) delete i->image;
			delete i;
		}			
	}

	bool autodelete() { return autodelete_; }
	void autodelete(bool val) { autodelete_ = val; }
	int size() { return size_; }
	void size(int s) { size_ = s; check_size(); }
	int cached() { return images.count(); }
};

#endif

